/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver;

import java.lang.annotation.Annotation;

import javassist.CannotCompileException;
import javassist.CtBehavior;
import javassist.CtClass;
import javassist.CtMethod;
import jp.objectfanatics.assertion.constraints.ConstraintAnnotation;

/**
 * Assertion weaver for a constraint annotation.
 * 
 * @author Makoto Sato
 */
public interface ConstraintAssertionWeaver {
	
	/**
	 * Returns target constraint annotation. It must be annotated with {@link ConstraintAnnotation}.
	 * 
	 * @return target constraint annotation
	 */
	Class<? extends Annotation> getTargetConstraintAnnotation();
	
	/**
	 * Weaves constraint assertion for return type.
	 * 
	 * @param ctMethod
	 *            target method
	 * @param returnType
	 *            return type
	 * @throws CannotCompileException
	 */
	void weaveReturnTypeConstraintAssertion(CtMethod ctMethod, CtClass returnType, javassist.bytecode.annotation.Annotation annotation);
	
	/**
	 * Weaves constraint assertion for a parameter.
	 * 
	 * @param nonAbstractCtBehavior
	 *            a handle for dealing with the target constructor or method. It must be non-abstract.
	 * @param paramType
	 *            target parameter type
	 * @param paramIndex
	 *            target parameter index
	 * @param paramValueIdentifier
	 *            parameter value identifier for javassist language
	 * @throws CannotCompileException
	 */
	void weaveParameterConstraintAssertion(CtBehavior nonAbstractCtBehavior, CtClass paramType, int paramIndex, String paramValueIdentifier, javassist.bytecode.annotation.Annotation annotation);
}
