/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.constraint;

import static jp.objectfanatics.commons.javassist.JavassistUtils.getLineNumberEvenIfAvailableLineNumberInfoNotAvailable;
import javassist.CannotCompileException;
import javassist.CtBehavior;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.bytecode.annotation.Annotation;
import jp.objectfanatics.assertion.constraints.NotNull;
import jp.objectfanatics.assertion.weaver.ConstraintAssertionWeaver;
import jp.objectfanatics.assertion.weaver.Weaver;
import jp.objectfanatics.assertion.weaver.WeaverUtil;

/**
 * Implementation of {@link ConstraintAssertionWeaver} for @{@link NotNull} annotation.
 * 
 * @author Makoto Sato
 */
public class NotNullAssertionWeaver extends AbstractAssertionWeaver {
	
	/**
	 * Constructor.
	 */
	public NotNullAssertionWeaver() {
		super(NotNull.class);
	}
	
	@Override
	protected void validateParameterConstraint(CtBehavior ctBehavior, CtClass paramType, int paramIndex) {
		// weaving is invalid for primitive type parameter
		String parameterName = WeaverUtil.getParameterNamesEvenIfNoAvailableParameterNameInfo(ctBehavior)[paramIndex];
		int lineNumber = getLineNumberEvenIfAvailableLineNumberInfoNotAvailable(ctBehavior);
		if (paramType.isPrimitive()) throw new Weaver.NoSuitableAnnotatedParameterTypeException(getTargetConstraintAnnotation(), ctBehavior, paramType, paramIndex, parameterName, lineNumber);
	}
	
	@Override
	protected void doWeaveParameterConstraintAssertion(CtBehavior ctBehavior, CtClass paramType, int paramIndex, String paramValueIdentifier, Annotation annotation) {
		try {
		// weaves
		String assertionErrorCondition = paramValueIdentifier + " == null";
		String message = "parameter " + paramIndex + "(zero base)" + " is null."; // TODO: use parameter name in the message.
		String assertionStatement = generateParameterAssrtionStatement(assertionErrorCondition, message);
		ctBehavior.insertBefore(assertionStatement);
		} catch (CannotCompileException e) {
			throw new IllegalStateException(e.getMessage(), e);
		}
	}
	
	@Override
	protected void validateReturnTypeConstraint(CtMethod ctMethod, CtClass returnType) {
		// weaving is invalid for primitive type parameter
		int lineNumber = getLineNumberEvenIfAvailableLineNumberInfoNotAvailable(ctMethod);
		if (returnType.isPrimitive()) throw new Weaver.NoSuitableAnnotatedReturnTypeException(getTargetConstraintAnnotation(), ctMethod, returnType, lineNumber);
	}
	
	@Override
	protected void doWeaveReturnTypeConstraintAssertion(CtMethod ctMethod, CtClass paramType, Annotation annotation) {
		try {
			// weaves
			String assertionErrorCondition = "$_ == null";
			String message = "return value is null.";
			String assertionStatement = generateReturnValueAssrtionStatement(assertionErrorCondition, message);
			ctMethod.insertAfter(assertionStatement);
		}
		catch (CannotCompileException e) {
			throw new IllegalStateException(e.getMessage(), e);
		}
	}
}
